﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Runtime.Remoting.Messaging;

namespace AsyncOperationCallbackDelegate
{
    public delegate int NetworkOperation();

    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Asynchronous Operation with Callback");
            Console.WriteLine("Main() thread {0}.",
                Thread.CurrentThread.GetHashCode());

            NetworkOperation netOp = new NetworkOperation(UploadFirmware);
            IAsyncResult iftAR = netOp.BeginInvoke( 
                new AsyncCallback(OperationComplete), 
                "After network operation: Now back in Main().");

            Console.WriteLine("Upload is running - do more work now!");
            
            for(int i = 1; i <= 5; i++)
			{
				Console.WriteLine("Parallel work in Main() - Item {0} of {1}", i, 5);
				Thread.Sleep(1000);
			}
            
            Console.ReadLine();
        }

        static void OperationComplete(IAsyncResult itfAR)
        {
            Console.WriteLine("OperationComplete() thread {0}.",
                Thread.CurrentThread.GetHashCode());
            Console.WriteLine("Firmware upload is complete");

            // Retrieve the operation result.
            AsyncResult ar = (AsyncResult)itfAR;
            NetworkOperation netOp = (NetworkOperation)ar.AsyncDelegate;
            Console.WriteLine("Blocks uploaded: {0}.", netOp.EndInvoke(itfAR));

            // Retrieve the informational object and cast it to string
            string msg = (string)itfAR.AsyncState;
            Console.WriteLine(msg);
        }

        static int UploadFirmware()
        {
			const int interBlockDelay = 1500;
			int numberOfBlocks = 5;
			
        	Console.WriteLine("UploadFirmware() thread {0}.",
                Thread.CurrentThread.GetHashCode());
		
            // Do the content download.
			Console.WriteLine("Now executing the upload block transfers");
			for(int i = 1; i <= numberOfBlocks; i++)
			{
				Console.WriteLine("Uploading on Thread {0} Block {1} of {2}",
				                  Thread.CurrentThread.GetHashCode(), i, numberOfBlocks);
				Thread.Sleep(interBlockDelay);
			}

            return numberOfBlocks;
        }
    }
}
